(document:surround "/std/frame")

;;; Helpers for checklistbox
(define *prev-current-rows* (make-cell #f))

(define (include-parent-items?) #t)

(define (checklistbox-current)
  (let* ((cr (packages current-rows))
	 (pcr (cell-ref *prev-current-rows*)))
    (cond-car (if pcr (lset-difference = cr pcr) cr))))

(define (checklistbox-update-current)
  (cell-set! *prev-current-rows* (packages current-rows)))

(define (checklistbox-revert-current)
  (packages current-rows (cell-ref *prev-current-rows*)))

(define (checklistbox-state num)
  (list-ref (packages current-rows) num))

;;; Features: list of packages
(define *pkg-lists* (make-cell '()))

(define (make-pkg-lists pkg-groups)
  (cell-set! *pkg-lists* (map (lambda(x) (woo-get-option x 'lists)) pkg-groups)))

; Lookup field value by id and field name
(define (pkg-lookup-field pkg-groups id field)
  (if (not-empty-string? id)
    (woo-get-option 
      (car 
        (filter (lambda (x) (string=? (woo-get-option x 'name) id)) pkg-groups))
	  field)))

;;; Features: Profiles
(define (pkg-profile-fill)
  (form-update-enum
    "profile"
	(woo-list "/pkg-groups" 'objects "profiles")))

(define (pkg-profile-groups profile)
  (if (not-empty-string? profile)
    (filter (lambda (y) (pkg-exists? y))
      (string-split
		(woo-get-option
		  (car
			(filter (lambda (x)
				(string=? (woo-get-option x 'name) profile))
                (woo-list "/pkg-groups" 'objects "profiles")))
		  'groups)
		#\ ))))

(define (pkg-profile-apply)
  (packages current-rows
  ;(format #t "PROFILE: ~S\n"
	(pkg-profile-groups (cadr (form-value-list '("profile"))))))

;;; Features: Lists

(define *groups-list* (make-cell #f))

; Return parent items name as list
(define (get-parents x)
  (if
    (not-empty-string? x)
	  (cons x (get-parents (pkg-lookup-field (cell-ref *groups-list*) x 'parent)))
	  '()))

; Get checked items identifiers
(define (get-checked-rows)
  (if
    (include-parent-items?)
      ; make unique list of checked items with their parents
	  (delete-duplicates!
	    (append-map
		  get-parents
		  (packages current-rows)))
      ; return only complete checked items
	  (packages current-rows)))

; Return list of checked group lists
(define (pkg-install-list)
  (if
    (proper-list? (packages current-rows))
 	(string-join 
 		(map 
 		(lambda (x) (pkg-lookup-field (cell-ref *groups-list*) x 'lists))
		(get-checked-rows))
 		" ")))

; Check if package with specified name is exists
(define (pkg-exists? name)
  (and
    (not-empty-string? name)
    (any 
      (lambda (x) (string=? (woo-get-option x 'name) name))
      (cell-ref *groups-list*))))

;;; Features: Depends - relation to keep both item checked

; Mark all depended item for current checked
; Need to check wrong package names in dependency list
(define (pkg-depends x)
  (filter (lambda (y) (pkg-exists? y))
    (string-split
      (pkg-lookup-field (cell-ref *groups-list*) x 'depends)
      #\ )))

; Mark checked all items depended on currently checked
(define (pkg-marks-depended)
  (if 
    (proper-list? (packages current-rows))
    (packages current-rows
    ;(format #t "CHECKED: ~S\n"
       (filter (lambda (x) (not-empty-string? x))
         (apply append 
           (map (lambda (pkg) (append (list pkg) (pkg-depends pkg)))
             (packages current-rows)))
    ))))

;;; Features: Conflicts - relation to prevent both item checked

(define *known-conflicts* (make-cell '()))

; Get conflict list for item
(define (pkg-conflict-list x)
  (filter (lambda (y) (pkg-exists? y))
    (string-split
      (pkg-lookup-field (cell-ref *groups-list*) x 'conflicts)
      #\ )))

; Check if conflicted item is checked
(define (pkg-conflicts? pkg)
  (and
    (any (lambda (x) (string=? pkg x)) (packages current-rows)) ; conflicted item is checked?
    (not (any (lambda (x)
      (or
        (string=? pkg (car x)) ; conflicted item is not in conflict list?
        (string=? pkg (cdr x))))
      (cell-ref *known-conflicts*)))
  ))

; Check if conflicted items checked and display dialog box
(define (pkg-check-conflicts pkg)
  (for-each (lambda (conflicted)
    (if
      (pkg-conflicts? conflicted)
      (cell-set! *known-conflicts* (append (cell-ref *known-conflicts*) (list (cons pkg conflicted))))))
      ;(format #t "CONFLICT between ~S and ~S\n" pkg conflicted)))
    (pkg-conflict-list pkg)))

; Search unresolved conflicts
(define (pkg-search-conflicts)
  ; Empty conflict list
  (cell-set! *known-conflicts* '())

  ; Fill list
  (for-each (lambda (pkg) (pkg-check-conflicts pkg))
    (packages current-rows))

  ; If conflicts is detected, show dialog
  (if (not (null? (cell-ref *known-conflicts*)))
      ; Conflict list is not empty: show it
      (document:popup-warning
        (string-append
          "<p align=left><b>"
          (_ "There are conflicts detected:")
          "</b><br><br>"
          (string-join
            (map (lambda (x)
              (string-append
                " "
                (pkg-lookup-field (cell-ref *groups-list*) (car x) 'label)
                (_ " and ")
                (pkg-lookup-field (cell-ref *groups-list*) (cdr x) 'label)))
              (cell-ref *known-conflicts*))
             ", ")
          "</p>")
        'ok)))

;; Generate rows content and hierarchy for tree. Prepare data tuple in CheckTree
; format of rows method. See http://www.altlinux.org/Alterator/CheckTree#API 
; for details
(define (row-content x)
  (list 
	(woo-get-option x 'name) 
	(woo-get-option x 'parent)
	(woo-get-option x 'label) 	
  ))

; Check item state. 
; Return flag if group is explicitly marked as required in directory file.
(define (is-checked x)
  (or
    (string=? (woo-get-option x 'required) "yes")
    (string=? (woo-get-option x 'required) "true")))

; Generate checked item list
(define (row-checked x)
  (woo-get-option x 'name))


;;; Features: Disk usage

(define (string-prefix-at-pos str prefix pos)
  (and (string-prefix? prefix str)
       (list-ref (string-cut str #\:) pos)))

; Generate installed software size in human readable format
; TODO need to add space between number and units
(define (pkg-size-string msg)
  (if (string-null? msg)
    msg
    (let* ((len (- (string-length msg) 1))
	   (first-msg (substring msg 0 len))
	   (last-msg (string-ref msg len)))
      (string-append
	first-msg
	(case last-msg
	  ((#\k) (_ " kB"))
	  ((#\M) (_ " MB"))
	  ((#\G) (_ " GB"))
	  ((#\T) (_ " TB"))
	  (else (string last-msg)))))))
	
(define (pkg-size-label msg)
  (size text (format #f "~A: ~A" (_ "Required disk space") msg)))

(define (pkg-message-handler msg)
  (and-let* ((msg (cond-plistq 'message msg #f)))
	    (cond
	      ((string=? msg "pkg-size:start")
	       (pkg-size-label (_ "calculating...")))
	      ((string-prefix-at-pos msg "apt-get:status:disk-size:" 3)
	        => (lambda(msg) (pkg-size-label (pkg-size-string msg))))
	      ((string-prefix-at-pos msg "pkg-size:finish" 2) =>
	        (lambda(result)
		  (if (string=? result "100") 
		    (begin 
		      (pkg-size-label (_ "unable to install in selected configuration"))
		      (frame:next-activity #f)
		    )
		    (frame:next-activity #t)))))))


(define (pkg-error-handler msg)
  (and-let* ((error-text (cond-plistq 'error msg #f)))
	    (format #t "alterator-pkg: ~S~%" error-text)))

(define (make-pkg-size)
  (add-mailbox pkg-message-handler)
  (add-mailbox pkg-error-handler)
  (pkg-size-label "0 kB"))

(define (remove-pkg-size)
  (remove-mailbox pkg-message-handler)
  (remove-mailbox pkg-error-handler)
  (woo-write "/pkg-size" 'lists "" 'language (form-value "language")))

(define (check-pkg-size)
  ;(format #f "INSTALLED: ~S\n" (pkg-install-list)))
  (woo-write "/pkg-size" 'lists (pkg-install-list) 'language (form-value "language")))

; Update group contents
(define (update-group-details group)
  (details text 
	   (woo-get-option (woo-read-first "/pkg-groups" 'group group) 'content)))

; Update details-pane contents
(define (details-pane-update)
  (form-update-visibility "details-pane" (form-value "show-details")))

;;; UI
(gridbox
  columns "1;3;1"

  ;; Select profiles
  (spacer)
  (gridbox visibility (not (null-list? (woo-list "/pkg-groups" 'objects "profiles")))
    columns "1;99"
    (label text (_ "Profile:"))
    (combobox name "profile"
              (when changed
                    (pkg-profile-apply)
                    (pkg-marks-depended)
                    (pkg-search-conflicts)
                    (check-pkg-size))))
  (spacer)

  ;;
  (spacer)
  (gridbox columns "0;0"
    (label text (_ "Additional applications:"))
    (label text (_ "Selected group contains:") name "details-pane" visibility #f))
  (spacer)

  ;;
  (spacer)
  (gridbox columns "0;0"
    ; Package group list
    (document:id packages (checktree min-height 400
                          (when changed
                                (pkg-marks-depended)
                                (pkg-search-conflicts)
                                (check-pkg-size))
                          (when selected
                                (update-group-details (packages current)))))

    ; Additional information about group
    (document:id details (textbox text "" name "details-pane" visibility #f)))
  (spacer)
  ;;
  (spacer)
  (document:id size (label text (_ "Preparing...")))
  (spacer)
  ;;
  (spacer)
  (checkbox
    name "show-details"
    text (_ "Show group details")
    state (woo-get-option
            (woo-read-first "/pkg-groups" 'objects "pkg_groups-config") 'expand_description)
    (when toggled (details-pane-update)))
  (spacer))

;;; Logic

(frame:on-next (thunk (remove-pkg-size)
		      (frame:replace "/pkg/install" 'lists (pkg-install-list))
		      'cancel))

(frame:on-back (thunk (remove-pkg-size)))

(document:root
  (when loaded
    (let ((state (form-value "show-details")))
        (if (eq? state #t) (details-pane-update)))
    (woo-catch/message
      (thunk
        (frame:back-activity #f)
		(woo-write "pkg-init")
       ; Read list data from /var/lib/install3/groups/*.directory
	(let* ((pkg-groups (woo-list "/pkg-groups")))
     (if (null? pkg-groups)
        (frame:replace "/pkg/install")
	(begin

	  ; Fill profile list
	  (pkg-profile-fill)

	  ; Fill packages list
	  (make-pkg-lists pkg-groups)
	  (cell-set! *groups-list*  pkg-groups)
		  
	  ; Calculate installed software size
	  (make-pkg-size)
	      
	  ; Fill group tree rows
	  (packages rows (map row-content pkg-groups))
	  ; Set checked item state
	  (packages current-rows (map row-checked (filter is-checked pkg-groups)))

          ; Mark all depended group
          (pkg-marks-depended)
          
	  ; Unmark all conflicted items
	  (pkg-search-conflicts)

          ; Inital size of installation
          (check-pkg-size)
)))))))
